/*
 * SMB.java
 *
 * Created on 02 January 2007, 13:37
 *
 */

package org.apache.nutch.protocol.smb;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.UTF8;
import org.apache.nutch.crawl.CrawlDatum;
import org.apache.nutch.net.protocols.HttpDateFormat;
import org.apache.nutch.protocol.Protocol;
import org.apache.nutch.protocol.ProtocolOutput;
import org.apache.nutch.protocol.ProtocolStatus;
import org.apache.nutch.util.NutchConfiguration;


/**
 *
 * @author Armel T. Nene
 */
public class SMB implements Protocol{
    
    public static final Log LOG = LogFactory.getLog(SMB.class);
    static final int MAX_REDIRECTS = 5;
    int maxContentLength;
    HttpDateFormat httpDateFormat = null;
    private Configuration conf;
    
    
    /** Creates a new instance of SMB */
    public SMB() {
        this.httpDateFormat = new HttpDateFormat();
    }

    /**
     * This method returns the protocol output after the fetching the URL.
     * @param url, crawl datum
     */
    public ProtocolOutput getProtocolOutput(UTF8 url, CrawlDatum datum) {
        String urlString = url.toString();
        System.out.println("SMB Protocol: Now using the SMB protocol");
        System.out.println("SMB Protocol will interrogate the following URL: "+urlString);
        try{
            URL u = new URL(urlString);
            int redirects = 0;
            if(!setJCifsProp())
                return new ProtocolOutput(null, new ProtocolStatus(new Exception("Properties file not found")));
            
            while(true){
                SMBResponse response;
                response = new SMBResponse(u, datum, this, getConf()); // make a request
                int code = response.getCode();
                
                if(code == 200){
                    return new ProtocolOutput(response.toContent());
                } else if (code >= 300 && code < 400){
                    if(redirects == MAX_REDIRECTS)
                        throw new SMBException("too many redirects: "+url);
                    u = new URL(response.getHeader("Location"));
                    redirects++;
                    if(LOG.isTraceEnabled())
                        LOG.trace("redirect to "+u);
                    else
                        throw new SMBError(code);
                }
            }
        }catch (Exception ex)
        {
            return new ProtocolOutput(null, new ProtocolStatus(ex));
        }
    }

    /**
     * This method set the configuration properties
     * @param Configuration
     */
    public void setConf(Configuration conf) {
        this.conf = conf;
    }

    /**
     * This method returns the configuration
     * @return configuration
     */
    public Configuration getConf() {
        return conf;
    }
    
    /**
     * This method sets the maximum content length of a file
     * @param length - the max length
     */ 
    public void setMaxContentLength(int length){
        this.maxContentLength = length;
    }
    
    /**
     * Set the properties for the JCIF protocol to run properly and returns true
     * if the properties were set without any problem
     * @return the success value - True | False
     */
    public boolean setJCifsProp() {
        InputStream is = NutchConfiguration.create().getConfResourceAsInputStream("smb.properties");
//        Properties prop = new Properties();
//        prop.load(is);
        try{
        jcifs.Config.load(is);
        return true;
        }catch (Exception ex){
            LOG.fatal("Could not load configuration file", ex);
            return false;
        }
    }
}
